/* 
 * CSE142 Homework 2 sample solution
 * V. Razmov and H. Perkins.  4/00
 */

#include <stdio.h>
#include <assert.h>

/* Maximum weight for any of homework, midterm, and final */
#define MAX_WEIGHT 100

/* Total weight of homework, midterm, and final */
#define TOTAL_WEIGHTS 100

/* Limit on largest score for any one assignment or exam */
#define MAX_ITEM_SCORE 1000

/* Maximum possible weighted score */
#define MAX_FINAL_SCORE 100.0

/* Number of assignments */
#define NUMBER_HWS 4

/* Max and min passing grades, and failing grade */
#define PERFECT_GRADE		4.0
#define MIN_PASSING_GRADE	0.7
#define FAILING_GRADE		0.0


/* = min(a,b) */
int min(int a, int b) {
	if (a < b)
		return a;
	else
		return b;
}

/* Exit if nRead != 1 (input failed) or if val is */
/* not in the range min_val <= val <= max_val     */
void exit_if_bad_input(int nRead, double val, double min_val, double max_val) {
	if (nRead != 1) {
		printf("error: input operation failed.\n");
		assert(0);
	}
	if (val < min_val || val > max_val) {
		printf("error: input value out of range.\n");
		assert(0);
	}
}


/* Read and return an integer from input.  Abort if the */
/* input operation fails or the value is <min or >max   */
int getInt(int min, int max) {
	int val, scanned;

	scanned = scanf("%d", &val);
	exit_if_bad_input(scanned, val, min, max);
	return val;
}


/* Read and return a double from input.  Abort if the */
/* input operation fails or the value is <min or >max */
double getDouble(double min, double max) {
	double val;
	int scanned;

	scanned = scanf("%lf", &val);
	exit_if_bad_input(scanned, val, min, max);
	return val;
}


/* given scores for hw1, hw2, hw3, and hw4 and the max points for any homework, */
/* return a normalized homework score in the range 0.0 to 1.0.                  */
double homeworkScore(int hw1, int hw2, int hw3, int hw4, int hw_max)
{
    double norm_score;
	int sum_scores, min_score;

	sum_scores = hw1 + hw2 + hw3 + hw4;
	min_score  = min(min(hw1, hw2) , min(hw3, hw4));
    norm_score = (double)(sum_scores - min_score) / ((NUMBER_HWS - 1) * hw_max);

    return norm_score;
}


/* given min passing and min perfect scores and student's score, */
/* calculate and return the corresponding final grade. */
double finalGrade(double low_cutoff, double high_cutoff, double weighted_score)
{
    double grade;

    if (weighted_score >= high_cutoff) 
		grade = PERFECT_GRADE;
	else if (weighted_score < low_cutoff) 
		grade = FAILING_GRADE;
	else grade = MIN_PASSING_GRADE +  (PERFECT_GRADE - MIN_PASSING_GRADE) *
						(weighted_score - low_cutoff) / (high_cutoff - low_cutoff);
	return grade;
}

/* main function */
int main(void)
{	
	int hw_max, midterm_max, final_max;				/* max scores on individual hw or exam	*/
	int hw1_score, hw2_score, hw3_score, hw4_score;	/* homework raw scores					*/
	int midterm_score, final_score;					/* exam raw scores						*/
	double hw_norm_score, midterm_norm_score,		/* normalized homework and exam scores	*/
			final_norm_score;
	int hw_weight, midterm_weight, final_weight;	/* component weights in final score		*/
	double weighted_score;							/* final score							*/
	double high_cutoff;								/* minimum score for perfect grade		*/
	double low_cutoff;								/* minimum passing score				*/
	double final_grade;								/* calculated grade						*/
	
	/* read and verify weights for assignments and exams */
	printf("Enter integer weights between 0 and %d.  Weights must sum up to %d.\n",
			MAX_WEIGHT, TOTAL_WEIGHTS);

	printf("homework: ");
	hw_weight = getInt(0,MAX_WEIGHT);

	printf("midterm: ");
	midterm_weight = getInt(0,MAX_WEIGHT);

	printf("final: ");
	final_weight = getInt(0,MAX_WEIGHT);

	if(hw_weight + midterm_weight + final_weight != TOTAL_WEIGHTS) {
		printf("error: weights do not add up to %d.\n", TOTAL_WEIGHTS);
		assert(0);
	}

	/* read and verify max scores for individual homeworks, midterm, and final */
	printf("\nEnter maximum scores between 0 and %d.\n", MAX_ITEM_SCORE);

	printf("homework: ");
	hw_max = getInt(0,MAX_ITEM_SCORE);

	printf("midterm: ");
	midterm_max = getInt(0,MAX_ITEM_SCORE);

	printf("final: ");
	final_max = getInt(0,MAX_ITEM_SCORE);

	/* read and verify min passing and min perfect grade scores */
	printf("\nEnter grade cutoffs between 0.0 and %.1f.\n", MAX_FINAL_SCORE);
	printf("minimum passing score: ");
	low_cutoff = getDouble(0.0 ,MAX_FINAL_SCORE);
	printf("minimum score for perfect grade: ");
	high_cutoff = getDouble(0.0, MAX_FINAL_SCORE);

	if(low_cutoff >= high_cutoff) {
		printf("error: low cutoff > high cutoff\n");
		assert(0);
	}
	
	/* read student scores for assignments and exams */
	printf("\nEnter scores for %d homework assignments (each %d max): ", NUMBER_HWS, hw_max);
	hw1_score = getInt(0,hw_max);
	hw2_score = getInt(0,hw_max);
	hw3_score = getInt(0,hw_max);
	hw4_score = getInt(0,hw_max);
	
	printf("Enter midterm exam score (%d max): ", midterm_max);
	midterm_score = getInt(0, midterm_max);
	
	printf("Enter final exam score (%d max): ", final_max);
	final_score = getInt(0, final_max);
	
	/* Calculate normalized scores (between 0.0 and 1.0) */
	hw_norm_score      = homeworkScore(hw1_score, hw2_score, hw3_score, hw4_score, hw_max);
	midterm_norm_score = (double)midterm_score / midterm_max;
	final_norm_score   = (double)final_score / final_max;
	
	/* Calculate and report weighted score (between 0.0 and 100.0) */
	weighted_score =  hw_weight * hw_norm_score + 
					  midterm_weight * midterm_norm_score +
					  final_weight * final_norm_score;
	printf("\nFinal score is %.1f\n", weighted_score);
	
	/* Calculate and report final grade	 */
	final_grade = finalGrade(low_cutoff, high_cutoff, weighted_score);
	printf("Final grade is %.1f\n\n", final_grade);
	
	return 0;
}